@@ -11,7 +11,7 @@ import sys
 import threading
 import unicodedata
 from enum import Enum
-from typing import Any, Callable, Iterable, List, Optional, TextIO, Union
+from typing import Any, Callable, Dict, Iterable, List, Optional, TextIO, Union
 
 from . import constants
 
@@ -72,6 +72,31 @@ def str_to_bool(val: str) -> bool:
     raise ValueError("must be True or False (case-insensitive)")
 
 
+class CompletionError(Exception):
+    """
+    Raised during tab completion operations to report any sort of error you want printed by the ArgparseCompleter
+    This can also be used just to display a message, even if it's not an error. ArgparseCompleter raises
+    CompletionErrors to display tab completion hints and sets apply_style to False so hints aren't colored
+    like error text.
+
+    Example use cases
+    - Reading a database to retrieve a tab completion data set failed
+    - A previous command line argument that determines the data set being completed is invalid
+    - Tab completion hints
+    """
+    def __init__(self, *args, apply_style: bool = True, **kwargs):
+        """
+        Initializer for CompletionError
+        :param apply_style: If True, then ansi.style_error will be applied to the message text when printed.
+                            Set to False in cases where the message text already has the desired style.
+                            Defaults to True.
+        """
+        self.apply_style = apply_style
+
+        # noinspection PyArgumentList
+        super().__init__(*args, **kwargs)
+
+
 class Settable:
     """Used to configure a cmd2 instance member to be settable via the set command in the CLI"""
     def __init__(self, name: str, val_type: Callable, description: str, *,
@@ -104,13 +129,13 @@ class Settable:
         :param choices: iterable of accepted values
         :param choices_function: function that provides choices for this argument
         :param choices_method: cmd2-app method that provides choices for this argument (See note below)
-        :param completer_function: tab-completion function that provides choices for this argument
-        :param completer_method: cmd2-app tab-completion method that provides choices
+        :param completer_function: tab completion function that provides choices for this argument
+        :param completer_method: cmd2-app tab completion method that provides choices
                                  for this argument (See note below)
 
         Note:
-        For choices_method and completer_method, do not set them to a bound method. This is because AutoCompleter
-        passes the self argument explicitly to these functions.
+        For choices_method and completer_method, do not set them to a bound method. This is because
+        ArgparseCompleter passes the self argument explicitly to these functions.
 
         Therefore instead of passing something like self.path_complete, pass cmd2.Cmd.path_complete.
         """
@@ -660,7 +685,7 @@ class RedirectionSavedState:
 def basic_complete(text: str, line: str, begidx: int, endidx: int, match_against: Iterable) -> List[str]:
     """
     Basic tab completion function that matches against a list of strings without considering line contents
-    or cursor position. The args required by this function are defined in the header of Pythons's cmd.py.
+    or cursor position. The args required by this function are defined in the header of Python's cmd.py.
 
     :param text: the string prefix we are attempting to match (all matches must begin with it)
     :param line: the current input line with leading whitespace removed
@@ -682,8 +707,8 @@ def align_text(text: str, alignment: TextAlignment, *, fill_char: str = ' ',
                width: Optional[int] = None, tab_width: int = 4, truncate: bool = False) -> str:
     """
     Align text for display within a given width. Supports characters with display widths greater than 1.
-    ANSI style sequences are safely ignored and do not count toward the display width. This means colored text is
-    supported. If text has line breaks, then each line is aligned independently.
+    ANSI style sequences do not count toward the display width. If text has line breaks, then each line is aligned
+    independently.
 
     There are convenience wrappers around this function: align_left(), align_center(), and align_right()
 
@@ -696,7 +721,7 @@ def align_text(text: str, alignment: TextAlignment, *, fill_char: str = ' ',
     :param truncate: if True, then each line will be shortened to fit within the display width. The truncated
                      portions are replaced by a '…' character. Defaults to False.
     :return: aligned text
-    :raises: TypeError if fill_char is more than one character
+    :raises: TypeError if fill_char is more than one character (not including ANSI style sequences)
              ValueError if text or fill_char contains an unprintable character
              ValueError if width is less than 1
     """
@@ -716,7 +741,7 @@ def align_text(text: str, alignment: TextAlignment, *, fill_char: str = ' ',
     if fill_char == '\t':
         fill_char = ' '
 
-    if len(fill_char) != 1:
+    if len(ansi.strip_style(fill_char)) != 1:
         raise TypeError("Fill character must be exactly one character long")
 
     fill_char_width = ansi.style_aware_wcswidth(fill_char)
@@ -777,8 +802,8 @@ def align_left(text: str, *, fill_char: str = ' ', width: Optional[int] = None,
                tab_width: int = 4, truncate: bool = False) -> str:
     """
     Left align text for display within a given width. Supports characters with display widths greater than 1.
-    ANSI style sequences are safely ignored and do not count toward the display width. This means colored text is
-    supported. If text has line breaks, then each line is aligned independently.
+    ANSI style sequences do not count toward the display width. If text has line breaks, then each line is aligned
+    independently.
 
     :param text: text to left align (can contain multiple lines)
     :param fill_char: character that fills the alignment gap. Defaults to space. (Cannot be a line breaking character)
@@ -788,7 +813,7 @@ def align_left(text: str, *, fill_char: str = ' ', width: Optional[int] = None,
     :param truncate: if True, then text will be shortened to fit within the display width. The truncated portion is
                      replaced by a '…' character. Defaults to False.
     :return: left-aligned text
-    :raises: TypeError if fill_char is more than one character
+    :raises: TypeError if fill_char is more than one character (not including ANSI style sequences)
              ValueError if text or fill_char contains an unprintable character
              ValueError if width is less than 1
     """
@@ -800,8 +825,8 @@ def align_center(text: str, *, fill_char: str = ' ', width: Optional[int] = None
                  tab_width: int = 4, truncate: bool = False) -> str:
     """
     Center text for display within a given width. Supports characters with display widths greater than 1.
-    ANSI style sequences are safely ignored and do not count toward the display width. This means colored text is
-    supported. If text has line breaks, then each line is aligned independently.
+    ANSI style sequences do not count toward the display width. If text has line breaks, then each line is aligned
+    independently.
 
     :param text: text to center (can contain multiple lines)
     :param fill_char: character that fills the alignment gap. Defaults to space. (Cannot be a line breaking character)
@@ -811,7 +836,7 @@ def align_center(text: str, *, fill_char: str = ' ', width: Optional[int] = None
     :param truncate: if True, then text will be shortened to fit within the display width. The truncated portion is
                      replaced by a '…' character. Defaults to False.
     :return: centered text
-    :raises: TypeError if fill_char is more than one character
+    :raises: TypeError if fill_char is more than one character (not including ANSI style sequences)
              ValueError if text or fill_char contains an unprintable character
              ValueError if width is less than 1
     """
@@ -823,8 +848,8 @@ def align_right(text: str, *, fill_char: str = ' ', width: Optional[int] = None,
                 tab_width: int = 4, truncate: bool = False) -> str:
     """
     Right align text for display within a given width. Supports characters with display widths greater than 1.
-    ANSI style sequences are safely ignored and do not count toward the display width. This means colored text is
-    supported. If text has line breaks, then each line is aligned independently.
+    ANSI style sequences do not count toward the display width. If text has line breaks, then each line is aligned
+    independently.
 
     :param text: text to right align (can contain multiple lines)
     :param fill_char: character that fills the alignment gap. Defaults to space. (Cannot be a line breaking character)
@@ -834,7 +859,7 @@ def align_right(text: str, *, fill_char: str = ' ', width: Optional[int] = None,
     :param truncate: if True, then text will be shortened to fit within the display width. The truncated portion is
                      replaced by a '…' character. Defaults to False.
     :return: right-aligned text
-    :raises: TypeError if fill_char is more than one character
+    :raises: TypeError if fill_char is more than one character (not including ANSI style sequences)
              ValueError if text or fill_char contains an unprintable character
              ValueError if width is less than 1
     """
@@ -845,8 +870,15 @@ def align_right(text: str, *, fill_char: str = ' ', width: Optional[int] = None,
 def truncate_line(line: str, max_width: int, *, tab_width: int = 4) -> str:
     """
     Truncate a single line to fit within a given display width. Any portion of the string that is truncated
-    is replaced by a '…' character. Supports characters with display widths greater than 1. ANSI style sequences are
-    safely ignored and do not count toward the display width. This means colored text is supported.
+    is replaced by a '…' character. Supports characters with display widths greater than 1. ANSI style sequences
+    do not count toward the display width.
+
+    If there are ANSI style sequences in the string after where truncation occurs, this function will append them
+    to the returned string.
+
+    This is done to prevent issues caused in cases like: truncate_string(fg.blue + hello + fg.reset, 3)
+    In this case, "hello" would be truncated before fg.reset resets the color from blue. Appending the remaining style
+    sequences makes sure the style is in the same state had the entire string been printed.
 
     :param line: text to truncate
     :param max_width: the maximum display width the resulting string is allowed to have
@@ -855,6 +887,7 @@ def truncate_line(line: str, max_width: int, *, tab_width: int = 4) -> str:
     :raises: ValueError if text contains an unprintable character like a new line
              ValueError if max_width is less than 1
     """
+    import io
     from . import ansi
 
     # Handle tabs
@@ -866,12 +899,95 @@ def truncate_line(line: str, max_width: int, *, tab_width: int = 4) -> str:
     if max_width < 1:
         raise ValueError("max_width must be at least 1")
 
-    if ansi.style_aware_wcswidth(line) > max_width:
-        # Remove characters until we fit. Leave room for the ellipsis.
-        line = line[:max_width - 1]
-        while ansi.style_aware_wcswidth(line) > max_width - 1:
-            line = line[:-1]
+    if ansi.style_aware_wcswidth(line) <= max_width:
+        return line
 
-        line += "\N{HORIZONTAL ELLIPSIS}"
+    # Find all style sequences in the line
+    styles = get_styles_in_text(line)
 
-    return line
+    # Add characters one by one and preserve all style sequences
+    done = False
+    index = 0
+    total_width = 0
+    truncated_buf = io.StringIO()
+
+    while not done:
+        # Check if a style sequence is at this index. These don't count toward display width.
+        if index in styles:
+            truncated_buf.write(styles[index])
+            style_len = len(styles[index])
+            styles.pop(index)
+            index += style_len
+            continue
+
+        char = line[index]
+        char_width = ansi.style_aware_wcswidth(char)
+
+        # This char will make the text too wide, add the ellipsis instead
+        if char_width + total_width >= max_width:
+            char = constants.HORIZONTAL_ELLIPSIS
+            char_width = ansi.style_aware_wcswidth(char)
+            done = True
+
+        total_width += char_width
+        truncated_buf.write(char)
+        index += 1
+
+    # Append remaining style sequences from original string
+    truncated_buf.write(''.join(styles.values()))
+
+    return truncated_buf.getvalue()
+
+
+def get_styles_in_text(text: str) -> Dict[int, str]:
+    """
+    Return an OrderedDict containing all ANSI style sequences found in a string
+
+    The structure of the dictionary is:
+        key: index where sequences begins
+        value: ANSI style sequence found at index in text
+
+    Keys are in ascending order
+
+    :param text: text to search for style sequences
+    """
+    from . import ansi
+
+    start = 0
+    styles = collections.OrderedDict()
+
+    while True:
+        match = ansi.ANSI_STYLE_RE.search(text, start)
+        if match is None:
+            break
+        styles[match.start()] = match.group()
+        start += len(match.group())
+
+    return styles
+
+
+def categorize(func: Union[Callable, Iterable[Callable]], category: str) -> None:
+    """Categorize a function.
+
+    The help command output will group the passed function under the
+    specified category heading
+
+    :param func: function or list of functions to categorize
+    :param category: category to put it in
+
+    :Example:
+
+    >>> class MyApp(cmd2.Cmd):
+    >>>   def do_echo(self, arglist):
+    >>>     self.poutput(' '.join(arglist)
+    >>>
+    >>>   utils.categorize(do_echo, "Text Processing")
+
+    For an alternative approach to categorizing commands using a decorator, see
+    :func:`~cmd2.decorators.with_category`
+    """
+    if isinstance(func, Iterable):
+        for item in func:
+            setattr(item, constants.CMD_ATTR_HELP_CATEGORY, category)
+    else:
+        setattr(func, constants.CMD_ATTR_HELP_CATEGORY, category)
